@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 //
-//  Copyright (C) 2006-2018 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2006-2021 Fons Adriaensen <fons@linuxaudio.org>
 //    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 
 
 #include <endian.h>
+#include <stdint.h>
+#include <byteswap.h>
 #include <sys/time.h>
 #include "zita-alsa-pcmi.h"
 
@@ -49,7 +51,7 @@ Alsa_pcmi::Alsa_pcmi (const char        *play_name,
     _fsize (fsize),
     _nfrag (nfrag),   
     _debug (debug),
-    _state (-1),
+    _state (STATE_FAIL),
     _play_handle (0),
     _capt_handle (0),
     _ctrl_handle (0),
@@ -78,11 +80,10 @@ Alsa_pcmi::~Alsa_pcmi (void)
     if (_play_handle) snd_pcm_close (_play_handle);
     if (_capt_handle) snd_pcm_close (_capt_handle);
     if (_ctrl_handle) snd_ctl_close (_ctrl_handle); 
-
-    snd_pcm_sw_params_free (_capt_swpar);
-    snd_pcm_hw_params_free (_capt_hwpar);
-    snd_pcm_sw_params_free (_play_swpar);
-    snd_pcm_hw_params_free (_play_hwpar);
+    if (_capt_swpar) snd_pcm_sw_params_free (_capt_swpar);
+    if (_capt_hwpar) snd_pcm_hw_params_free (_capt_hwpar);
+    if (_play_swpar) snd_pcm_sw_params_free (_play_swpar);
+    if (_play_hwpar) snd_pcm_hw_params_free (_play_hwpar);
 }
 
 
@@ -93,10 +94,18 @@ int Alsa_pcmi::pcm_start (void)
 
     if (_play_handle)
     {
+        if ((err = snd_pcm_prepare (_play_handle)) < 0)
+        {
+            if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_prepare(play): %s\n",
+                                              snd_strerror (err));
+	    _state = STATE_FAIL;
+            return -1;
+	}
         n = snd_pcm_avail_update (_play_handle);
         if (n != _fsize * _nfrag)
         {
             if (_debug & DEBUG_STAT) fprintf  (stderr, "Alsa_pcmi: full buffer not available at start.\n");
+            _state = STATE_FAIL;
             return -1;
         }
         for (i = 0; i < _nfrag; i++)
@@ -108,15 +117,26 @@ int Alsa_pcmi::pcm_start (void)
         if ((err = snd_pcm_start (_play_handle)) < 0)
         {
             if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_start(play): %s.\n", snd_strerror (err));
+	    _state = STATE_FAIL;
             return -1;
         }
     }
-    if (_capt_handle && !_synced && ((err = snd_pcm_start (_capt_handle)) < 0))
+    if (_capt_handle && !_synced)
     {
-        if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_start(capt): %s.\n", snd_strerror (err));
-        return -1;
+        if ((err = snd_pcm_prepare (_capt_handle)) < 0)
+        {
+            if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_prepare(capt): %s\n",
+                                              snd_strerror (err));
+	    _state = STATE_FAIL;
+            return -1;
+	}
+	if ((err = snd_pcm_start (_capt_handle)) < 0)
+        {
+            if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_start(capt): %s.\n", snd_strerror (err));
+            _state = STATE_FAIL;
+            return -1;
+	}
     }
-
     return 0;
 }
 
@@ -128,14 +148,15 @@ int Alsa_pcmi::pcm_stop (void)
     if (_play_handle && ((err = snd_pcm_drop (_play_handle)) < 0))
     {
         if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_drop(play): %s.\n", snd_strerror (err));
+	_state = STATE_FAIL;
         return -1;
     }
     if (_capt_handle && !_synced && ((err = snd_pcm_drop (_capt_handle)) < 0))
     {
         if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_drop(capt): %s.\n", snd_strerror (err));
+	_state = STATE_FAIL;
         return -1;
     }
-
     return 0;
 }
 
@@ -149,7 +170,8 @@ snd_pcm_sframes_t Alsa_pcmi::pcm_wait (void)
     unsigned short    rev;
     int               i, r, n1, n2;
 
-    _state = 0;
+    if (_state != STATE_OPEN) return 0;
+    
     need_capt = _capt_handle ? true : false;
     need_play = _play_handle ? true : false;
 
@@ -174,13 +196,11 @@ snd_pcm_sframes_t Alsa_pcmi::pcm_wait (void)
         {
             if (errno == EINTR) return 0;
             if (_debug & DEBUG_WAIT) fprintf (stderr, "Alsa_pcmi: poll(): %s\n.", strerror (errno));
-            _state = -1;
             return 0;
         }
         if (r == 0)
         { 
             if (_debug & DEBUG_WAIT) fprintf (stderr, "Alsa_pcmi: poll timed out.\n");
-            _state = -1;
             return 0;
         }               
 
@@ -190,7 +210,6 @@ snd_pcm_sframes_t Alsa_pcmi::pcm_wait (void)
             if (rev & POLLERR)
             {
                 if (_debug & DEBUG_WAIT) fprintf (stderr, "Alsa_pcmi: error on playback pollfd.\n");
-                _state = 1;
                 recover ();
                 return 0;
             }
@@ -202,7 +221,6 @@ snd_pcm_sframes_t Alsa_pcmi::pcm_wait (void)
             if (rev & POLLERR)
             {
                 if (_debug & DEBUG_WAIT) fprintf (stderr, "Alsa_pcmi: error on capture pollfd.\n");
-                _state = 1;
                 recover ();
                 return 0;
             }
@@ -213,18 +231,15 @@ snd_pcm_sframes_t Alsa_pcmi::pcm_wait (void)
     play_av = 999999999;
     if (_play_handle && (play_av = snd_pcm_avail_update (_play_handle)) < 0)
     {
-        _state = -1;
         recover ();
         return 0;
     }
     capt_av = 999999999;
     if (_capt_handle && (capt_av = snd_pcm_avail_update (_capt_handle)) < 0)
     {
-        _state = -1;
         recover ();
         return 0;
     }
-
     return (capt_av < play_av) ? capt_av : play_av;
 }
 
@@ -268,7 +283,7 @@ int Alsa_pcmi::play_init (snd_pcm_uframes_t len)
     if ((err = snd_pcm_mmap_begin (_play_handle, &a, &_play_offs, &len)) < 0)
     {
         if (_debug & DEBUG_DATA) fprintf (stderr, "Alsa_pcmi: snd_pcm_mmap_begin(play): %s.\n", snd_strerror (err)); 
-        return -1;
+        return 0;
     }
     _play_step = (a->step) >> 3;
     for (i = 0; i < _play_nchan; i++, a++)
@@ -289,7 +304,7 @@ int Alsa_pcmi::capt_init (snd_pcm_uframes_t len)
     if ((err = snd_pcm_mmap_begin (_capt_handle, &a, &_capt_offs, &len)) < 0)
     {
         if (_debug & DEBUG_DATA) fprintf (stderr, "Alsa_pcmi: snd_pcm_mmap_begin(capt): %s.\n", snd_strerror (err)); 
-        return -1;
+        return 0;
     }
     _capt_step = (a->step) >> 3;
 
@@ -463,90 +478,62 @@ void Alsa_pcmi::initialise (const char *play_name, const char *capt_name, const
         snd_pcm_hw_params_get_format (_play_hwpar, &_play_format);
         snd_pcm_hw_params_get_access (_play_hwpar, &_play_access);
 
-#if __BYTE_ORDER == __LITTLE_ENDIAN
         switch (_play_format)
         {
         case SND_PCM_FORMAT_FLOAT_LE:
             _clear_func = &Alsa_pcmi::clear_32;     
-            _play_func  = &Alsa_pcmi::play_float;     
-            break;
-
-        case SND_PCM_FORMAT_S32_LE:
-            _clear_func = &Alsa_pcmi::clear_32;     
-            _play_func  = &Alsa_pcmi::play_32;     
-            break;
-
-        case SND_PCM_FORMAT_S32_BE:
-            _clear_func = &Alsa_pcmi::clear_32;     
-            _play_func  = &Alsa_pcmi::play_32swap;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3LE:
-            _clear_func = &Alsa_pcmi::clear_24;     
-            _play_func  = &Alsa_pcmi::play_24;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3BE:
-            _clear_func = &Alsa_pcmi::clear_24;     
-            _play_func  = &Alsa_pcmi::play_24swap;     
-            break;
-
-        case SND_PCM_FORMAT_S16_LE:
-            _clear_func = &Alsa_pcmi::clear_16;     
-            _play_func  = &Alsa_pcmi::play_16;     
-            break;
-
-        case SND_PCM_FORMAT_S16_BE:
-            _clear_func = &Alsa_pcmi::clear_16;     
-            _play_func  = &Alsa_pcmi::play_16swap;     
-            break;
-
-        default:
-            if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: can't handle playback sample format.\n"); 
-            return;
-        }
-#elif __BYTE_ORDER == __BIG_ENDIAN
-        switch (_play_format)
-        {
-        case SND_PCM_FORMAT_S32_LE:
-            _clear_func = &Alsa_pcmi::clear_32;     
-            _play_func  = &Alsa_pcmi::play_32swap;     
-            break;
-
-        case SND_PCM_FORMAT_S32_BE:
-            _clear_func = &Alsa_pcmi::clear_32;     
-            _play_func  = &Alsa_pcmi::play_32;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3LE:
-            _clear_func = &Alsa_pcmi::clear_24;     
-            _play_func  = &Alsa_pcmi::play_24swap;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3BE:
-            _clear_func = &Alsa_pcmi::clear_24;     
-            _play_func  = &Alsa_pcmi::play_24;     
-            break;
-
-        case SND_PCM_FORMAT_S16_LE:
-            _clear_func = &Alsa_pcmi::clear_16;     
-            _play_func  = &Alsa_pcmi::play_16swap;     
-            break;
-
-        case SND_PCM_FORMAT_S16_BE:
-            _clear_func = &Alsa_pcmi::clear_16;     
-            _play_func  = &Alsa_pcmi::play_16;     
-            break;
-
-        default:
-            if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: can't handle playback sample format.\n"); 
-            return;
-        }
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+            _play_func  = &Alsa_pcmi::play_floatne;     
 #else
-#error "System byte order is undefined or not supported"
-#endif
+            _play_func  = &Alsa_pcmi::play_floatre;     
+#endif            
+            break;
 
-       _play_npfd = snd_pcm_poll_descriptors_count (_play_handle);
+        case SND_PCM_FORMAT_FLOAT_BE:
+            _clear_func = &Alsa_pcmi::clear_32;     
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+            _play_func  = &Alsa_pcmi::play_floatre;     
+#else
+            _play_func  = &Alsa_pcmi::play_floatne;     
+#endif            
+            break;
+
+        case SND_PCM_FORMAT_S32_LE:
+            _clear_func = &Alsa_pcmi::clear_32;     
+            _play_func  = &Alsa_pcmi::play_32le;     
+            break;
+
+        case SND_PCM_FORMAT_S32_BE:
+            _clear_func = &Alsa_pcmi::clear_32;     
+            _play_func  = &Alsa_pcmi::play_32be;
+            break;
+
+        case SND_PCM_FORMAT_S24_3LE:
+            _clear_func = &Alsa_pcmi::clear_24;     
+            _play_func  = &Alsa_pcmi::play_24le;     
+            break;
+
+        case SND_PCM_FORMAT_S24_3BE:
+            _clear_func = &Alsa_pcmi::clear_24;     
+            _play_func  = &Alsa_pcmi::play_24be;
+            break;
+
+        case SND_PCM_FORMAT_S16_LE:
+            _clear_func = &Alsa_pcmi::clear_16;     
+            _play_func  = &Alsa_pcmi::play_16le;     
+            break;
+
+        case SND_PCM_FORMAT_S16_BE:
+            _clear_func = &Alsa_pcmi::clear_16;     
+            _play_func  = &Alsa_pcmi::play_16be;     
+            break;
+
+        default:
+            if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: can't handle playback sample format.\n"); 
+            return;
+        }
+
+        _play_npfd = snd_pcm_poll_descriptors_count (_play_handle);
     }
 
     if (_capt_handle)
@@ -572,75 +559,52 @@ void Alsa_pcmi::initialise (const char *play_name, const char *capt_name, const
         snd_pcm_hw_params_get_format (_capt_hwpar, &_capt_format);
         snd_pcm_hw_params_get_access (_capt_hwpar, &_capt_access);
 
-#if __BYTE_ORDER == __LITTLE_ENDIAN
         switch (_capt_format)
         {
         case SND_PCM_FORMAT_FLOAT_LE:
-            _capt_func  = &Alsa_pcmi::capt_float;     
-            break;
-
-        case SND_PCM_FORMAT_S32_LE:
-            _capt_func  = &Alsa_pcmi::capt_32;     
-            break;
-
-        case SND_PCM_FORMAT_S32_BE:
-            _capt_func  = &Alsa_pcmi::capt_32swap;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3LE:  // 3 lower bytes of 32-bit word 
-            _capt_func  = &Alsa_pcmi::capt_24;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3BE:  // 3 lower bytes of 32-bit word
-            _capt_func  = &Alsa_pcmi::capt_24swap;     
-            break;
-
-        case SND_PCM_FORMAT_S16_LE:
-            _capt_func  = &Alsa_pcmi::capt_16;     
-            break;
-
-        case SND_PCM_FORMAT_S16_BE:
-            _capt_func  = &Alsa_pcmi::capt_16swap;     
-            break;
-
-        default:
-            if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: can't handle capture sample format.\n"); 
-            return;
-        }
-#elif __BYTE_ORDER == __BIG_ENDIAN
-        switch (_capt_format)
-        {
-        case SND_PCM_FORMAT_S32_LE:
-            _capt_func  = &Alsa_pcmi::capt_32swap;     
-            break;
-
-        case SND_PCM_FORMAT_S32_BE:
-            _capt_func  = &Alsa_pcmi::capt_32;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3LE:  // 3 lower bytes of 32-bit word
-            _capt_func  = &Alsa_pcmi::capt_24swap;     
-            break;
-
-        case SND_PCM_FORMAT_S24_3BE:  // 3 lower bytes of 32-bit word
-            _capt_func  = &Alsa_pcmi::capt_24;     
-            break;
-
-        case SND_PCM_FORMAT_S16_LE:
-            _capt_func  = &Alsa_pcmi::capt_16swap;     
-            break;
-
-        case SND_PCM_FORMAT_S16_BE:
-            _capt_func  = &Alsa_pcmi::capt_16;     
-            break;
-
-        default:
-            if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: can't handle capture sample format.\n"); 
-            return;
-        }
-#else
-#error "System byte order is undefined or not supported"
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+            _capt_func  = &Alsa_pcmi::capt_floatne;
+#else            
+            _capt_func  = &Alsa_pcmi::capt_floatre;
 #endif
+            break;
+
+        case SND_PCM_FORMAT_FLOAT_BE:
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+            _capt_func  = &Alsa_pcmi::capt_floatre;
+#else            
+            _capt_func  = &Alsa_pcmi::capt_floatne;
+#endif
+            break;
+
+        case SND_PCM_FORMAT_S32_LE:
+            _capt_func  = &Alsa_pcmi::capt_32le;     
+            break;
+
+        case SND_PCM_FORMAT_S32_BE:
+            _capt_func  = &Alsa_pcmi::capt_32be;     
+            break;
+
+        case SND_PCM_FORMAT_S24_3LE:
+            _capt_func  = &Alsa_pcmi::capt_24le;     
+            break;
+
+        case SND_PCM_FORMAT_S24_3BE:
+            _capt_func  = &Alsa_pcmi::capt_24be;     
+            break;
+
+        case SND_PCM_FORMAT_S16_LE:
+            _capt_func  = &Alsa_pcmi::capt_16le;     
+            break;
+
+        case SND_PCM_FORMAT_S16_BE:
+            _capt_func  = &Alsa_pcmi::capt_16be;     
+            break;
+
+        default:
+            if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: can't handle capture sample format.\n"); 
+            return;
+        }
 
         _capt_npfd = snd_pcm_poll_descriptors_count (_capt_handle);
     }
@@ -651,7 +615,7 @@ void Alsa_pcmi::initialise (const char *play_name, const char *capt_name, const
         return;
     }      
 
-    _state = 0;
+    _state = STATE_OPEN;
 }       
 
 
@@ -687,6 +651,7 @@ int Alsa_pcmi::set_hwpar (snd_pcm_t *handle,  snd_pcm_hw_params_t *hwpar, const
     else
     {
         err =    (snd_pcm_hw_params_set_format (handle, hwpar, SND_PCM_FORMAT_FLOAT_LE) < 0)
+              && (snd_pcm_hw_params_set_format (handle, hwpar, SND_PCM_FORMAT_FLOAT_BE) < 0)
               && (snd_pcm_hw_params_set_format (handle, hwpar, SND_PCM_FORMAT_S32_LE) < 0)
               && (snd_pcm_hw_params_set_format (handle, hwpar, SND_PCM_FORMAT_S32_BE) < 0)
               && (snd_pcm_hw_params_set_format (handle, hwpar, SND_PCM_FORMAT_S24_3LE) < 0)
@@ -696,7 +661,7 @@ int Alsa_pcmi::set_hwpar (snd_pcm_t *handle,  snd_pcm_hw_params_t *hwpar, const
     }
     if (err)
     {
-        if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: no supported sample format on %s interface.\n.",
+        if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: no supported sample format on %s interface.\n",
                                           sname);
         return -1;
     }
@@ -806,7 +771,7 @@ int Alsa_pcmi::recover (void)
         if ((err = snd_pcm_status (_play_handle, stat)) < 0)
         {
             if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_status(play): %s\n",
-                                              snd_strerror (err));
+                                             snd_strerror (err));
         }
         _play_xrun = xruncheck (stat);
     }
@@ -820,22 +785,7 @@ int Alsa_pcmi::recover (void)
         _capt_xrun = xruncheck (stat);
     }
 
-    if (pcm_stop ()) return -1;
-    if (_play_handle && ((err = snd_pcm_prepare (_play_handle)) < 0))
-    {
-        if (_debug & DEBUG_STAT) fprintf (stderr, "Alsa_pcmi: pcm_prepare(play): %s\n",
-                                          snd_strerror (err));
-        return -1;
-    }
-    if (_capt_handle && !_synced && ((err = snd_pcm_prepare (_capt_handle)) < 0))
-    {
-        if (_debug & DEBUG_INIT) fprintf (stderr, "Alsa_pcmi: pcm_prepare(capt): %s\n",
-                                          snd_strerror (err));
-        return -1;
-    }
-    if (pcm_start ()) return -1;
-
-    return 0;
+    return pcm_stop () || pcm_start ();
 }       
 
 
@@ -894,7 +844,7 @@ char *Alsa_pcmi::clear_32 (char *dst, int nfrm)
 }
 
 
-char *Alsa_pcmi::play_16 (const float *src, char *dst, int nfrm, int step)
+char *Alsa_pcmi::play_16le (const float *src, char *dst, int nfrm, int step)
 {
     float     s;
     short int d;
@@ -905,14 +855,15 @@ char *Alsa_pcmi::play_16 (const float *src, char *dst, int nfrm, int step)
         if      (s >  1) d = 0x7fff;
         else if (s < -1) d = 0x8001;
         else d = (short int)((float) 0x7fff * s); 
-        *((short int *) dst) = d;
+        dst [0] = d;
+        dst [1] = d >> 8;
         dst += _play_step;
         src += step;
     } 
     return dst; 
 }
 
-char *Alsa_pcmi::play_16swap (const float *src, char *dst, int nfrm, int step)
+char *Alsa_pcmi::play_16be (const float *src, char *dst, int nfrm, int step)
 {
     float     s;
     short int d;
@@ -931,7 +882,7 @@ char *Alsa_pcmi::play_16swap (const float *src, char *dst, int nfrm, int step)
     return dst; 
 }
 
-char *Alsa_pcmi::play_24 (const float *src, char *dst, int nfrm, int step)
+char *Alsa_pcmi::play_24le (const float *src, char *dst, int nfrm, int step)
 {
     float   s;
     int     d;
@@ -951,7 +902,7 @@ char *Alsa_pcmi::play_24 (const float *src, char *dst, int nfrm, int step)
     return dst;
 }
 
-char *Alsa_pcmi::play_24swap (const float *src, char *dst, int nfrm, int step)
+char *Alsa_pcmi::play_24be (const float *src, char *dst, int nfrm, int step)
 {
     float   s;
     int     d;
@@ -971,7 +922,7 @@ char *Alsa_pcmi::play_24swap (const float *src, char *dst, int nfrm, int step)
     return dst;
 }
 
-char *Alsa_pcmi::play_32 (const float *src, char *dst, int nfrm, int step)
+char *Alsa_pcmi::play_32le (const float *src, char *dst, int nfrm, int step)
 {
     float   s;
     int     d;
@@ -982,14 +933,17 @@ char *Alsa_pcmi::play_32 (const float *src, char *dst, int nfrm, int step)
         if      (s >  1) d = 0x007fffff;
         else if (s < -1) d = 0x00800001;  
         else d = (int)((float) 0x007fffff * s); 
-        *((int *) dst) = d << 8;
+        dst [0] = 0;
+        dst [1] = d;
+        dst [2] = d >> 8;
+        dst [3] = d >> 16;
         dst += _play_step;
         src += step;
     }  
     return dst;
 }
 
-char *Alsa_pcmi::play_32swap (const float *src, char *dst, int nfrm, int step)
+char *Alsa_pcmi::play_32be (const float *src, char *dst, int nfrm, int step)
 {
     float   s;
     int     d;
@@ -1010,7 +964,7 @@ char *Alsa_pcmi::play_32swap (const float *src, char *dst, int nfrm, int step)
     return dst;
 }
 
-char *Alsa_pcmi::play_float (const float *src, char *dst, int nfrm, int step)
+char *Alsa_pcmi::play_floatne (const float *src, char *dst, int nfrm, int step)
 {
     while (nfrm--)
     {
@@ -1022,43 +976,54 @@ char *Alsa_pcmi::play_float (const float *src, char *dst, int nfrm, int step)
 }
 
 
-const char *Alsa_pcmi::capt_16 (const char *src, float *dst, int nfrm, int step)
+char *Alsa_pcmi::play_floatre (const float *src, char *dst, int nfrm, int step)
 {
-    float     d;
-    short int s;
+    float d;
+    
+    while (nfrm--)
+    {
+        d = *src;
+        *((float *) dst) = __bswap_32 (d);
+        dst += _play_step;
+        src += step;
+    }  
+    return dst;
+}
+
+
+const char *Alsa_pcmi::capt_16le (const char *src, float *dst, int nfrm, int step)
+{
+    int16_t s;
 
     while (nfrm--)
     {
-        s = *((short int *) src);
-        d = (float) s / (float) 0x7fff;  
-        *dst = d;
+        s =  (src [0] & 0xFF);
+        s += (src [1] & 0xFF) << 8;  
+        *dst = (float) s / (float) 0x7fff;  
         dst += step;
         src += _capt_step;
     }  
     return src;
 }
 
-const char *Alsa_pcmi::capt_16swap (const char *src, float *dst, int nfrm, int step)
+const char *Alsa_pcmi::capt_16be (const char *src, float *dst, int nfrm, int step)
 {
-    float     d;
-    short int s;
+    int16_t s;
 
     while (nfrm--)
     {
         s =  (src [0] & 0xFF) << 8;
         s += (src [1] & 0xFF);  
-        d = (float) s / (float) 0x7fff;  
-        *dst = d;
+        *dst = (float) s / (float) 0x7fff;  
         dst += step;
         src += _capt_step;
     }  
     return src;
 }
 
-const char *Alsa_pcmi::capt_24 (const char *src, float *dst, int nfrm, int step)
+const char *Alsa_pcmi::capt_24le (const char *src, float *dst, int nfrm, int step)
 {
-    float   d;
-    int     s;
+    int32_t s;
 
     while (nfrm--)
     {
@@ -1066,18 +1031,16 @@ const char *Alsa_pcmi::capt_24 (const char *src, float *dst, int nfrm, int step)
         s += (src [1] & 0xFF) << 8;  
         s += (src [2] & 0xFF) << 16;
         if (s & 0x00800000) s-= 0x01000000;  
-        d = (float) s / (float) 0x007fffff;  
-        *dst = d;
+        *dst = (float) s / (float) 0x007fffff;  
         dst += step;
         src += _capt_step;
     } 
     return src;
 }
 
-const char *Alsa_pcmi::capt_24swap (const char *src, float *dst, int nfrm, int step)
+const char *Alsa_pcmi::capt_24be (const char *src, float *dst, int nfrm, int step)
 {
-    float   d;
-    int     s;
+    int32_t s;
 
     while (nfrm--)
     {
@@ -1085,49 +1048,46 @@ const char *Alsa_pcmi::capt_24swap (const char *src, float *dst, int nfrm, int s
         s += (src [1] & 0xFF) << 8;  
         s += (src [2] & 0xFF);
         if (s & 0x00800000) s-= 0x01000000;  
-        d = (float) s / (float) 0x007fffff;  
-        *dst = d;
+        *dst = (float) s / (float) 0x007fffff;  
         dst += step;
         src += _capt_step;
     } 
     return src;
 }
 
-const char *Alsa_pcmi::capt_32 (const char *src, float *dst, int nfrm, int step)
+const char *Alsa_pcmi::capt_32le (const char *src, float *dst, int nfrm, int step)
 {
-    float   d;
-    int     s;
+    int32_t s;
 
     while (nfrm--)
     {
-        s = *((int *) src);
-        d = (float) s / (float) 0x7fffff00;  
-        *dst = d;  
+        s  = (src [0] & 0xFF) << 8;
+        s += (src [1] & 0xFF) << 16;  
+        s += (src [2] & 0xFF) << 24;
+        *dst = (float) s / (float) 0x7fffff00;  
         dst += step;
         src += _capt_step;
     }  
     return src;
 }
 
-const char *Alsa_pcmi::capt_32swap (const char *src, float *dst, int nfrm, int step)
+const char *Alsa_pcmi::capt_32be (const char *src, float *dst, int nfrm, int step)
 {
-    float   d;
-    int     s;
+    int32_t s;
 
     while (nfrm--)
     {
         s  = (src [0] & 0xFF) << 24;
         s += (src [1] & 0xFF) << 16;  
         s += (src [2] & 0xFF) << 8;
-        d = (float) s / (float) 0x7fffff00;  
-        *dst = d;  
+        *dst = (float) s / (float) 0x7fffff00;  
         dst += step;
         src += _capt_step;
     }  
     return src;
 }
 
-const char *Alsa_pcmi::capt_float (const char *src, float *dst, int nfrm, int step)
+const char *Alsa_pcmi::capt_floatne (const char *src, float *dst, int nfrm, int step)
 {
     while (nfrm--)
     {
@@ -1138,3 +1098,17 @@ const char *Alsa_pcmi::capt_float (const char *src, float *dst, int nfrm, int st
     return src;
 }
 
+const char *Alsa_pcmi::capt_floatre (const char *src, float *dst, int nfrm, int step)
+{
+    float d;
+    
+    while (nfrm--)
+    {
+        d = *((float *) src);
+        *dst = __bswap_32 (d);
+        dst += step;
+        src += _capt_step;
+    }  
+    return src;
+}
+
